package com.persagy.proxy.adm.service.impl;

import cn.hutool.core.collection.CollUtil;

import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.proxy.adm.constant.enums.AdmDictCategoryEnum;
import com.persagy.proxy.adm.model.*;
import com.persagy.proxy.adm.request.AdmCreateRequest;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IAdmDictService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName AdmDictServiceImpl
 * @Description: 字典查询
 * @Author linhuili
 * @Date 2021/8/31 18:21
 * @Version V1.0
 **/
@Service
@Slf4j
public class AdmDictServiceImpl<T> extends AbstractAdmDictBaseServiceImpl<T> implements IAdmDictService<T> {

    /**
     * 字典类型查询:所有设备类型、部件类型、系统类型、专业类型
     * 查询类型：输入设备类型：equipment，输入部件类型：component,输入系统类型：system,输入专业类型：major,业务空间类型 space ,资产 property,
     *         专业和系统：majorSystem（树）,设备和部件equipmentAndComponent,设备类型要以及对应的部件equipmentComponent(树)，
     *         专业下的设备类型 majorEquipment （树），专业下的系统，系统下的设备majorSystemEquip （树）,systemEquip 系统下的设备类（树）
     * @param request
     * @return
     * @throws Exception
     */
    @Override
    public AdmResponse queryCategory(InstanceUrlParam context , AdmDict request, Class<T> clazz) throws Exception {
        String type = request.getType();
        QueryCriteria criteria = new QueryCriteria();
        ObjectNode node = JsonNodeFactory.instance.objectNode();
        if(AdmDictCategoryEnum.MAJOR.getValue().equals(type)){
            //查询专业分类
            node.put("type",AdmDictCategoryEnum.MAJOR.getValue());
            criteria.setCriteria(node);
            return doQueryDictData(context, criteria, clazz);
        }

        List<String> baseCategory = getDictBaseCategory();
        if(baseCategory.contains(type)){
            //查询设备类型：equipment，部件类型：component,输入系统类型：system,输入专业类型：major,业务空间类型 space ,资产 property
            node.put("objType",type);
            criteria.setCriteria(node);
            return doQueryObjectType(context, criteria, clazz);
        }

        if(AdmDictCategoryEnum.EQUIPMENT_AND_COMPONENT.getValue().equals(type)){
            //查询设备类型：equipment，部件类型：component,输入系统类型：system,输入专业类型：major,业务空间类型 space ,资产 property
            List<String> objTypes = new ArrayList<>();
            objTypes.add(AdmDictCategoryEnum.EQUIPMENT.getValue());
            objTypes.add(AdmDictCategoryEnum.COMPONENT.getValue());
            node.put("objType",objTypes.toString());
            criteria.setCriteria(node);
            return doQueryObjectType(context, criteria, clazz);
        }
        List<String> dictTreeCategory = getDictTreeCategory();
        if(dictTreeCategory.contains(type)){
            //查询树形结构：专业和系统：majorSystem（树）,设备类型要以及对应的部件equipmentComponent(树)，专业下的设备类型 majorEquipment（树），
            //专业下的系统，系统下的设备majorSystemEquip （树）,systemEquip 系统下的设备类（树）
            return queryDictObjectTypeTree(context ,type,clazz);
        }
        return null;
    }


    /**
     * 查询字典树
     * @param context
     * @param type
     * @param clazz
     */
    private AdmResponse queryDictObjectTypeTree(InstanceUrlParam context ,String type, Class<T> clazz){
        //查询所有的专业
        List<AdmObjectType> allMajor = getAllMajor(context, clazz);

        //查询所有的类型
        AdmResponse admResponse = doQueryObjectType(context, new QueryCriteria(), clazz);
        List<AdmObjectType> objectTypes = (List<AdmObjectType>) admResponse.getContent();
        Map<String, List<AdmObjectType>> objectTypeMap = objectTypes.stream().filter(k-> StringUtils.isNotEmpty(k.getObjType()))
                                                         .collect(Collectors.groupingBy(AdmObjectType::getObjType));
        //获取专业-设备类关系
        List<AdmObjectType> equipments = objectTypeMap.get(AdmDictCategoryEnum.EQUIPMENT.getValue());
        Map<String,List<AdmObjectType>> majorEquipMap = equipments.stream().collect(Collectors.groupingBy(item -> item.getMajorCode()));

        //获取专业-系统类关系
        List<AdmObjectType> systems = objectTypeMap.get(AdmDictCategoryEnum.SYSTEM.getValue());
        Map<String, List<AdmObjectType>> majorSystemMap = systems.stream().collect(Collectors.groupingBy(item -> item.getMajorCode()));

        //获取系统-设备类关系
        Map<String, List<AdmObjectType>> systemEquipMap = equipments.stream().collect(Collectors.groupingBy(item -> item.getSystemCode()));

        //获取设备类-部件关系
        List<AdmObjectType> components = objectTypeMap.get(AdmDictCategoryEnum.COMPONENT.getValue());
        Map<String, List<AdmObjectType>> equipComponentMap = components.stream().collect(Collectors.groupingBy(item -> item.getEquipmentCode()));

        List<AdmObjectType> content = new ArrayList<>();
        if(AdmDictCategoryEnum.MAJOR_SYSTEM.getValue().equals(type)){
            //查询专业下的系统
            content = buildDictTree(allMajor,majorSystemMap,null);
        }
        if(AdmDictCategoryEnum.SYSTEM_EQUIP.getValue().equals(type)){
            //查询系统下的设备类
            content = buildDictTree(systems,systemEquipMap,null);
        }
        if(AdmDictCategoryEnum.MAJOR_SYSTEM_EQUIP.getValue().equals(type)){
            //专业下的系统，系统下的设备
            content = buildDictTree(allMajor,majorSystemMap,systemEquipMap);
        }
        if(AdmDictCategoryEnum.MAJOR_EQUIPMENT.getValue().equals(type)){
            //查询专业下的设备类
            content = buildDictTree(allMajor,majorEquipMap,null);
        }
        if(AdmDictCategoryEnum.EQUIPMENT_COMPONENT.getValue().equals(type)){
            //查询设备类下的部件
            content = buildDictTree(equipments,equipComponentMap,null);
        }
        // 设置返回值
        AdmResponse response = AdmResponse.success(content);
        response.setTotal(Long.valueOf(content.size()));
        return response;
    }

    /**
     * 构建专业-系统-设备类树形结构
     * @param allMajor
     * @param majorSystemMap
     * @param systemEquipMap
     * @return
     */
    private ArrayList<AdmObjectType> buildDictTree(List<AdmObjectType> allMajor,Map<String, List<AdmObjectType>> majorSystemMap,Map<String, List<AdmObjectType>> systemEquipMap){
        ArrayList<AdmObjectType> result = new ArrayList<>();
        if(CollUtil.isEmpty(allMajor)){
            return result;
        }
        //构建专业-系统-设备类树形结构
        for (AdmObjectType major : allMajor) {
             //设置专业
            if(CollUtil.isNotEmpty(majorSystemMap)){
                //专业下的系统
                List<AdmObjectType> systemCodes = majorSystemMap.get(major.getCode());
                major.setChildren(systemCodes);
                //系统下的设备类
                if(CollUtil.isNotEmpty(systemEquipMap) && CollUtil.isNotEmpty(systemCodes)){
                    for (AdmObjectType systemType : systemCodes) {
                        systemType.setChildren(systemEquipMap.get(systemType.getCode()));
                    }
                }
            }
            result.add(major);
        }
        return result;
    }

    /**
     * 获取所有的专业
     * @param context
     * @param clazz
     * @return
     */
    private List<AdmObjectType> getAllMajor(InstanceUrlParam context, Class<T> clazz){
        //查询专业分类
        QueryCriteria criteria = new QueryCriteria();
        ObjectNode node = JsonNodeFactory.instance.objectNode();
        node.put("type",AdmDictCategoryEnum.MAJOR.getValue());
        criteria.setCriteria(node);
        AdmResponse admResponse = doQueryDictData(context, criteria, clazz);
        List<AdmObjectType> content = (List<AdmObjectType>) admResponse.getContent();
        return content;
    }

    /**
     * 基础数据类型
     * @return
     */
    private List<String> getDictBaseCategory(){
        List<String> category = new ArrayList<>();
        category.add(AdmDictCategoryEnum.EQUIPMENT.getValue());
        category.add(AdmDictCategoryEnum.COMPONENT.getValue());
        category.add(AdmDictCategoryEnum.SYSTEM.getValue());
        category.add(AdmDictCategoryEnum.SPACE.getValue());
        category.add(AdmDictCategoryEnum.PROPRTY.getValue());
        return category;
    }

    /**
     * 查询字典树形结构
     * @return
     */
    private List<String> getDictTreeCategory(){
        List<String> category = new ArrayList<>();
        category.add(AdmDictCategoryEnum.MAJOR_SYSTEM.getValue());
        category.add(AdmDictCategoryEnum.MAJOR_EQUIPMENT.getValue());
        category.add(AdmDictCategoryEnum.MAJOR_SYSTEM_EQUIP.getValue());
        category.add(AdmDictCategoryEnum.EQUIPMENT_COMPONENT.getValue());
        category.add(AdmDictCategoryEnum.SYSTEM_EQUIP.getValue());
        return category;
    }

    /**
     * 重置项目下的信息点
     * @param context
     * @param clazz
     * @return
     * @throws Exception
     */
    @Override
    public AdmResponse resetDic(InstanceUrlParam context, AdmDictQueryCriteria request, String fieldName, Class<T> clazz) throws Exception {

        //查询项目下点所有信息点
        AdmResponse admResponse = doQueryObjectInfo(context, request, fieldName, clazz);
        List<AdmObjectInfo> content = (List<AdmObjectInfo>) admResponse.getContent();
        if(CollUtil.isEmpty(content)){
            AdmResponse response = AdmResponse.success(new ArrayList<>());
            response.setTotal(Long.valueOf(0));
            return response;
        }
        //因BDTP根据项目查询信息点时返回的包含集团+通用信息点，需手动过滤除项目级的信息点
        String projectId = context.getProjectId();

        Iterator<AdmObjectInfo> infoIterator = content.iterator();
        while (infoIterator.hasNext()){
            String admProjectId = infoIterator.next().getProjectId();
            if(StringUtils.isEmpty(admProjectId) || !projectId.equals(admProjectId)){
                infoIterator.remove();
            }
        }
        //删除信息点
        doDeleteObjectInfo(context,(List<T>)content);
        // 设置返回值
        AdmResponse response = AdmResponse.success(content);
        response.setTotal(Long.valueOf(content.size()));
        return response;
    }

    /**
     * 设置-项目定制化字典
     * @param context
     * @param request
     * @param clazz
     * @return
     * @throws Exception
     */
    @Override
    public AdmResponse settingDicProject(InstanceUrlParam context, AdmCreateRequest<AdmObjectInfo> request, Class<T> clazz) throws Exception {
        List<AdmObjectInfo> content = request.getContent();
        //信息点转换为中台格式
        List<AdmObjectInfo> dmpObjectInfo = new ArrayList<>();
        objectInfosTransforDmp(content, dmpObjectInfo ,context.getProjectId());
        request.setContent(dmpObjectInfo);

        //修改项目级信息点
        List<T> vos = doUpdateObjectInfo(context, clazz, (List<T>) dmpObjectInfo);
        return AdmResponse.success(vos);
    }

    /**
     * 信息点转换为中台格式
     * @param content
     * @param dmpObjectInfos
     */
    private void objectInfosTransforDmp(List<AdmObjectInfo> content,List<AdmObjectInfo> dmpObjectInfos,String projectId){
        if(CollUtil.isEmpty(content)){
            return;
        }
        for (AdmObjectInfo admObjectInfo : content) {
            admObjectInfo.setProjectId(projectId);
            if(admObjectInfo.getMultiple() != null){
                admObjectInfo.setIsMultiple(admObjectInfo.getMultiple());
            }
            if(admObjectInfo.getIsRegion() != null){
                admObjectInfo.setIsRegion(admObjectInfo.getRegion());
            }
            if(StringUtils.isNotEmpty(admObjectInfo.getFirstName())){
                admObjectInfo.setFirstTag(admObjectInfo.getFirstName());
            }
            if(StringUtils.isNotEmpty(admObjectInfo.getSecondName())){
                admObjectInfo.setSecondTag(admObjectInfo.getSecondName());
            }
            admObjectInfo.setType(null);
            dmpObjectInfos.add(admObjectInfo);
            //处理子级
            List<AdmObjectInfo> sencondContent = admObjectInfo.getContent();
            if(CollUtil.isNotEmpty(sencondContent)){
                objectInfosTransforDmp(sencondContent,dmpObjectInfos,projectId);
            }
        }
    }
}
