package com.persagy.proxy.adm.strategy;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.dmp.basic.dto.RequestData;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.strategy.relationdata.RelationObjectStrategy;

/**
 * 获取关系对象的上下文类
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年9月2日 下午10:44:32
 */
@Component
public class RelationObjectContext {

	/**
	 * 策略执行类
	 */
	private Map<String, RelationObjectStrategy> relationObjectStrategyMap;
	
	/**
	 * 自动注入 byName
	 * 
	 * @param relationObjectStrategyMap
	 */
	@Resource
	public void relationObjectStrategyMap(Map<String, RelationObjectStrategy> relationObjectStrategyMap) {
		this.relationObjectStrategyMap = relationObjectStrategyMap;
	}
	
	/**
	 * 获取关系对象数据
	 * 
	 * @param groupCode
	 * @param projectId
	 * @param relType
	 * @return
	 */
	public List<AdmRelationObject> findRelationObject(String groupCode, String projectId, String relType) {
		AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
		if (relationTypeEnum == null) {
			return Lists.newArrayList();
		}
		String className = relationTypeEnum.getGraphCode() + AdmCommonConstant.UNDERLINE + relationTypeEnum.getRelCode();
		RelationObjectStrategy strategy = this.relationObjectStrategyMap.get(className);
		return strategy == null ? Lists.newArrayList() : strategy.findRelationObject(groupCode, projectId);
	}
	
	/**
	 * 检查关联对象的合法性
	 * 
	 * @param relationObject 当前需要校验的对象
	 * @param relType ADM-类型
	 * @param code 获取对象的依据
	 * @return String - 校验失败的原因， ObjectNode -- BDTP接口的参数，不会返回null，请用instanceOf 判断返回值
	 */
	public Object checkRelationObject(AdmRelationObject relationObject, String groupCode, String projectId, String relType, String code) {
		AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
		if (relationTypeEnum == null) {
			return "未知类型";
		}
		String className = relationTypeEnum.getGraphCode() + AdmCommonConstant.UNDERLINE + relationTypeEnum.getRelCode();
		RelationObjectStrategy strategy = this.relationObjectStrategyMap.get(className);
		if (strategy == null) {
			return "不存在此策略类";
		}
		
		return strategy.beforeSaveRelationObject(relationObject, groupCode, projectId, code);
	}
	
	/**
	 * 批量保存对象之间的关系
	 * 
	 * @param admRelationObject
	 * @param groupCode
	 * @param projectId
	 * @param relType ADM-传过来的边类型
	 * @return 
	 */
	public boolean saveRelationObjects(List<ObjectNode> relationObjects, String groupCode, String projectId, String relType) {
		AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
		if (relationTypeEnum == null) {
			return false;
		}
		String className = relationTypeEnum.getGraphCode() + AdmCommonConstant.UNDERLINE + relationTypeEnum.getRelCode();
		RelationObjectStrategy strategy = this.relationObjectStrategyMap.get(className);
		if (strategy == null) {
			return false;
		}
		
		return strategy.saveRelationObjects(relationObjects, groupCode, projectId);
	}

	/**
	 * 统计关系总数
	 * 
	 * @param groupCode
	 * @param projectId
	 * @param relType ADM-传过来的边类型
	 * @return 
	 */
	public long countRelationObjects(String groupCode, String projectId, String relType) {
		AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
		if (relationTypeEnum == null) {
			return 0;
		}
		String className = relationTypeEnum.getGraphCode() + AdmCommonConstant.UNDERLINE + relationTypeEnum.getRelCode();
		RelationObjectStrategy strategy = this.relationObjectStrategyMap.get(className);
		if (strategy == null) {
			return 0;
		}
		
		return strategy.countRelationObjects(groupCode, projectId);
	}
	
	/**
	 * 分页查询关系对象数据
	 * 
	 * @param masterObjs 查询的数据集存储地方
	 * @param requestData 请求参数
	 * @param groupCode 集团编码
	 * @param projectId 项目ID
	 * @param relType 
	 */
	public List<ObjectDigital> queryPageObjectInfo(RequestData requestData, String groupCode, String projectId, String relType) {
		List<ObjectDigital> masterObjs = new ArrayList<ObjectDigital>();
		AdmRelationTypeEnum relationTypeEnum = AdmCommonConstant.RELATION_TYPE_MAP.get(relType);
		if (relationTypeEnum == null) {
			return masterObjs;
		}
		String className = relationTypeEnum.getGraphCode() + AdmCommonConstant.UNDERLINE + relationTypeEnum.getRelCode();
		RelationObjectStrategy strategy = this.relationObjectStrategyMap.get(className);
		if (strategy == null) {
			return masterObjs;
		}
		strategy.queryPageObjectInfo(masterObjs, requestData, groupCode, projectId, requestData.getPage());
		return masterObjs;
	}
	
}
